--
-- TatraTipper
-- Scripts for Tatra tippers
-- Based on file W60Kipper.lua by hz888
-- hz888@freemail.hu
--
-- @author  Tomas Kralicek (RabbiT_CZ)
-- @date  16/09/09
-- @version 1.0 CZ
--
-- t.kralicek@gmail.com
--

TatraTipper = {};

function TatraTipper.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function TatraTipper:load(xmlFile)

    -- Engine
    self.startMotor = SpecializationUtil.callSpecializationsFunction("startMotor");
    self.stopMotor = SpecializationUtil.callSpecializationsFunction("stopMotor");
    self.startRefuel = SpecializationUtil.callSpecializationsFunction("startRefuel");
    self.stopRefuel = SpecializationUtil.callSpecializationsFunction("stopRefuel");
    self.setFuelFillLevel = SpecializationUtil.callSpecializationsFunction("setFuelFillLevel");


    self.fuelCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fuelCapacity"), 500);
    self.fuelUsage = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fuelUsage"), 0.01);


    self.hasRefuelStationInRange = false;
    self.doRefuel = false;
    self:setFuelFillLevel(self.fuelCapacity);

    self.refuelSampleRunning = false;
    self.refuelSample = createSample("refuelSample");
    loadSample(self.refuelSample, "data/maps/sounds/refuel.wav", false);

    local motorMinRpm = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#minRpm"), 1000);
    local motorMaxRpmStr = getXMLString(xmlFile, "vehicle.motor#maxRpm");
    local motorMaxRpm1, motorMaxRpm2, motorMaxRpm3 = Utils.getVectorFromString(motorMaxRpmStr);
    motorMaxRpm1 = Utils.getNoNil(motorMaxRpm1, 800);
    motorMaxRpm2 = Utils.getNoNil(motorMaxRpm2, 1000);
    motorMaxRpm3 = Utils.getNoNil(motorMaxRpm3, 1800);
    local motorMaxRpm = {motorMaxRpm1, motorMaxRpm2, motorMaxRpm3};
    self.motorMaxRpmLimit = {motorMaxRpm1, motorMaxRpm2, motorMaxRpm3};
    local motorTorque = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#torque"), 15);
    local brakeForce = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#brakeForce"), 10)*2;
    local forwardGearRatioStr = getXMLString(xmlFile, "vehicle.motor#forwardGearRatio");
    local forwardGearRatio1, forwardGearRatio2, forwardGearRatio3 = Utils.getVectorFromString(forwardGearRatioStr);
    forwardGearRatio1 = Utils.getNoNil(forwardGearRatio1, 2);
    forwardGearRatio2 = Utils.getNoNil(forwardGearRatio2, forwardGearRatio1);
    forwardGearRatio3 = Utils.getNoNil(forwardGearRatio3, forwardGearRatio2);
    local forwardGearRatios = {forwardGearRatio1, forwardGearRatio2, forwardGearRatio3};
    local backwardGearRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#backwardGearRatio"), 1.5);
    local differentialRatio = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#differentialRatio"), 1);
    local rpmFadeOutRange = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motor#rpmFadeOutRange"), 20);
    local torqueCurve = AnimCurve:new(linearInterpolator1);
    local torqueI = 0;
    while true do
        local key = string.format("vehicle.motor.torque(%d)", torqueI);
        local rpm = getXMLFloat(xmlFile, key.."#rpm");
        local torque = getXMLFloat(xmlFile, key.."#torque");
        if torque == nil or rpm == nil then
            break;
        end;
        torqueCurve:addKeyframe({v=torque*3, time = rpm});
        torqueI = torqueI +1;
    end;
    self.motor = VehicleMotor:new(motorMinRpm, motorMaxRpm, torqueCurve, brakeForce, forwardGearRatios, backwardGearRatio, differentialRatio, rpmFadeOutRange);

    -- Engine sounds
	self.isFadingInMotorSndRun = false;
	self.fadeTime = 4000;
	self.currentFadeTime = 0;
	self.motorSoundActualVolume = 0;

    local motorStartSound = getXMLString(xmlFile, "vehicle.motorStartSound#file");
    if motorStartSound ~= nil and motorStartSound ~= "" then
        motorStartSound = Utils.getFilename(motorStartSound, self.baseDirectory);
        self.motorStartSound = createSample("motorStartSound");
        loadSample(self.motorStartSound, motorStartSound, false);
        self.motorStartSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStartSound#pitchOffset"), 0);
        self.motorStartSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStartSound#volume"), 1.0);
    end;

    local motorStopSound = getXMLString(xmlFile, "vehicle.motorStopSound#file");
    if motorStopSound ~= nil and motorStopSound ~= "" then
        motorStopSound = Utils.getFilename(motorStopSound, self.baseDirectory);
        self.motorStopSound = createSample("motorStopSound");
        loadSample(self.motorStopSound, motorStopSound, false);
        self.motorStopSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStopSound#pitchOffset"), 0);
        self.motorStopSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStopSound#volume"), 1.0);
    end;
    self.motorStopSoundVolumeBU = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorStopSound#volume"), 1.0);

    local motorSound = getXMLString(xmlFile, "vehicle.motorSound#file");
    if motorSound ~= nil and motorSound ~= "" then
        motorSound = Utils.getFilename(motorSound, self.baseDirectory);
        self.motorSound = createSample("motorSound");
        loadSample(self.motorSound, motorSound, false);
        self.motorSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchOffset"), 0);
        self.motorSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchScale"), 0.05);
        self.motorSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#pitchMax"), 2.0);
        self.motorSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSound#volume"), 1.0);
    end;

    local motorSoundRun = getXMLString(xmlFile, "vehicle.motorSoundRun#file");
    if motorSoundRun ~= nil and motorSoundRun ~= "" then
        motorSoundRun = Utils.getFilename(motorSoundRun, self.baseDirectory);
        self.motorSoundRun = createSample("motorSoundRun");
        loadSample(self.motorSoundRun, motorSoundRun, false);
        self.motorSoundRunPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#pitchOffset"), 0);
        self.motorSoundRunPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#pitchScale"), 0.05);
        self.motorSoundRunPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#pitchMax"), 2.0);
        self.motorSoundRunVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#volume"), 1.0);
    end;

    local compressionSound = getXMLString(xmlFile, "vehicle.compressionSound#file");
    if compressionSound ~= nil and compressionSound ~= "" then
        compressionSound = Utils.getFilename(compressionSound, self.baseDirectory);
        self.compressionSound = createSample("compressionSound");
        loadSample(self.compressionSound, compressionSound, false);
        self.compressionSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.compressionSound#pitchOffset"), 1);
        self.compressionSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.compressionSound#volume"), 1);
        self.compressionSoundTime = 0;
        self.compressionSoundEnabled = false;
    end;
    
    local compressedAirSound = getXMLString(xmlFile, "vehicle.compressedAirSound#file");
    if compressedAirSound ~= nil and compressedAirSound ~= "" then
        compressedAirSound = Utils.getFilename(compressedAirSound, self.baseDirectory);
        self.compressedAirSound = createSample("compressedAirSound");
        self.compressedAirSoundEnabled = false;
        loadSample(self.compressedAirSound, compressedAirSound, false);
        self.compressedAirSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.compressedAirSound#pitchOffset"), 1);
        self.compressedAirSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.compressedAirSound#volume"), 1);
    end;
    
    local aiMotorSound  = getXMLString(xmlFile, "vehicle.aiMotorSound#file");
    if aiMotorSound  ~= nil and aiMotorSound  ~= "" then
        aiMotorSound  = Utils.getFilename(aiMotorSound, self.baseDirectory);
        self.aiMotorSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#pitchOffset"), 0);
        self.aiMotorSoundRadius = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#radius"), 50);
        self.aiMotorSoundInnerRadius = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#innerRadius"), 10);
        self.aiMotorSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorSoundRun#volume"), 2);
        self.aiMotorSound = createAudioSource("aiMotorSound", aiMotorSound, self.aiMotorSoundRadius, self.aiMotorSoundInnerRadius, self.aiMotorSoundVolume, 0);
        link(self.components[1].node, self.aiMotorSound);
        setVisibility(self.aiMotorSound, false);
    end;
    
    local reverseDriveSound = getXMLString(xmlFile, "vehicle.reverseDriveSound#file");
    if reverseDriveSound ~= nil and reverseDriveSound ~= "" then
        reverseDriveSound = Utils.getFilename(reverseDriveSound, self.baseDirectory);
        self.reverseDriveSound = createSample("reverseDriveSound");
        self.reverseDriveSoundEnabled = false;
        loadSample(self.reverseDriveSound, reverseDriveSound, false);
        self.reverseDriveSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.reverseDriveSound#volume"), 1.0);
    end;
    

    self.isMotorStarted = false;

    self.exhaustParticleSystems = {};
    local exhaustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.exhaustParticleSystems#count"), 0);
    for i=1, exhaustParticleSystemCount do
        local namei = string.format("vehicle.exhaustParticleSystems.exhaustParticleSystem%d", i);
        Utils.loadParticleSystem(xmlFile, self.exhaustParticleSystems, namei, self.components, false, nil, self.baseDirectory)
    end;

    self.lastRoundPerMinute = 0;

    -- Steerable
    self.keys = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.keys.input(%d)", i);
        local inputName = getXMLString(xmlFile, baseName.. "#name");
        if inputName == nil then
            break;
        end;
        local inputKey = getXMLString(xmlFile, baseName.. "#key");
        if Input[inputKey] == nil then
            print("Error: invalid key '" .. inputKey .. "'  for input event '" .. inputName .. "'");
            break;
        end;
        self.keys[inputName] = Input[inputKey];
        i = i+1;
    end;

    self.onEnter = SpecializationUtil.callSpecializationsFunction("onEnter");
    self.onLeave = SpecializationUtil.callSpecializationsFunction("onLeave");
    self.drawGrainLevel = SpecializationUtil.callSpecializationsFunction("drawGrainLevel");


    self.enterReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.enterReferenceNode#index"));
    self.exitPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.exitPoint#index"));


    self.steering = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steering#index"));
    if self.steering ~= nil then
        self.steeringSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.steering#rotationSpeed"), 0);
    end;

    -- Cameras
    self.numCameras = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cameras#count"), 0);
    if self.numCameras == 0 then
        print("Error: no cameras in xml file: ", configFile);
    end;
    self.cameras = {};
    for i=1, self.numCameras do
        local cameranamei = string.format("vehicle.cameras.camera%d", i);
        local camIndexStr = getXMLString(xmlFile, cameranamei .. "#index");
        local cameraNode = Utils.indexToObject(self.components, camIndexStr);
        local rotatable = getXMLBool(xmlFile, cameranamei .. "#rotatable");
        local limit = getXMLBool(xmlFile, cameranamei .. "#limit");
        local rotMinX = getXMLFloat(xmlFile, cameranamei .. "#rotMinX");
        local rotMaxX = getXMLFloat(xmlFile, cameranamei .. "#rotMaxX");
        local transMin = getXMLFloat(xmlFile, cameranamei .. "#transMin");
        local transMax = getXMLFloat(xmlFile, cameranamei .. "#transMax");
        local rotateNode = "";
        if rotatable then
            rotateNode = Utils.indexToObject(self.components, getXMLString(xmlFile, cameranamei .. "#rotateNode"));
        end;
        self.cameras[i] = VehicleCamera:new(cameraNode, rotatable, rotateNode, limit, rotMinX, rotMaxX, transMin, transMax);
    end;

    self.camIndex = 1;

    self.tipCamera = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tipCamera#index"));
    
    -- Character node
    self.characterNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.characterNode#index"));
    if self.characterNode ~= nil then
        self.characterCameraMinDistance = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.characterNode#cameraMinDistance"), 1.5);
        setVisibility(self.characterNode, false);
    end;

    -- Lights
    self.numLights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.lights#count"), 0);
    self.lightObjects = {};
    for i=1, self.numLights do
        local lightnamei = string.format("vehicle.lights.light" .. "%d", i);
        self.lightObjects[i] = Utils.indexToObject(self.rootNode, getXMLInt(xmlFile, lightnamei .. "#index"));
        setVisibility(self.lightObjects[i], false);
    end;
    self.lightsActive = false;

    -- Low beam lights
    self.lowLightsObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.lowBeam#index"));
    setVisibility(self.lowLightsObject, false);
    self.lowLightsActive = false;
    
    -- High beam lights
    self.highLightsObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.highBeam#index"));
    setVisibility(self.highLightsObject, false);
    self.highLightsActive = false;
    
    -- Revers lights
    self.reverseLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.reverse#index"));
    setVisibility(self.reverseLightObject, false);
    self.reverseLightActive = false;
    
    -- Brake lights
    self.brakeLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.brake#index"));
    setVisibility(self.brakeLightObject, false);
    self.brakeLightActive = false;
    
    -- Turn lights
    self.leftTurnObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.leftTurn#index"))
    setVisibility(self.leftTurnObject, false);
    self.leftTurnActive = false;
    self.flashLeft = false;
    
    self.rightTurnObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.utilLights.rightTurn#index"))
    setVisibility(self.rightTurnObject, false);
    self.rightTurnActive = false;
    self.flashRight = false;
    
    self.flash = false;
    self.flashWarn = false;
    self.timer = 10;
    
    -- Dashboard
    self.rotateNeedle = SpecializationUtil.callSpecializationsFunction("rotateNeedle");
    self.PI = math.pi;
    self.PI_2 = math.pi / 2;
    --RPM
    self.rpmNeedle = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.dashboard.rpm#index"));
    self.rpmNeedleMin = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.rpm#min")));
    self.rpmNeedleMax = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.rpm#max")));
    self.rpmLastValue = 0;
    
    --Speed
    self.speedNeedle = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.dashboard.speed#index"));
    self.speedNeedleMin = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.speed#min")));
    self.speedNeedleMax = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.speed#max")));
    self.speedLastValue = 0;
    self.maxSpeed = 68;
    
    --Fuel
    self.fuelNeedle = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.dashboard.fuel#index"));
    self.fuelNeedleMin = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.fuel#min")));
    self.fuelNeedleMax = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.dashboard.fuel#max")));
    self.fuelLastValue = 0;    

    -- Other variables
    self.speedRotScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.speedRotScale#scale"), 80);
    self.speedRotScaleOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.speedRotScale#offset"), 0.7);
    self.isEntered = false;
    self.steeringEnabled = true;
    self.ignition = false;
    self.disableCharacterOnLeave = true;
    self.deactivateOnLeave = true;
    self.stopRefuelOnLeave = true;
    self.deactivateLightsOnLeave = true;
    self.showWaterWarning = false;
    self.waterSplashSample = nil;
    
    -- HUD
    self.hudBasePoxX = 0.8325;
    self.hudBasePoxY = (1-0.99);
    self.hudBaseWidth = 0.16;
    self.hudBaseHeight = 0.17;
    self.hudBaseOverlay = Overlay:new("hudBaseOverlay", "dataS/missions/hud_vehicle_base".. g_languageSuffix .. ".png", self.hudBasePoxX, self.hudBasePoxY, self.hudBaseWidth, self.hudBaseHeight);
    self.logoFile = Utils.getFilename("textures/Tatra_logo.png", self.baseDirectory);
    self.hudLogoWidth = 0.0312;
    self.hudLogoHeight = 0.05;
    self.hudLogoPoxX = 0.075;
    self.hudLogoPoxY = 0.435;
    self.hudLogoOverlay = Overlay:new("hudLogoOverlay", self.logoFile, self.hudLogoPoxX, self.hudLogoPoxY, self.hudLogoWidth, self.hudLogoHeight);
end;

function TatraTipper:delete()
    for i=1, table.getn(self.cameras) do
        self.cameras[i]:delete();
    end;

    if self.hudBaseOverlay then
        self.hudBaseOverlay:delete();
    end;
    
    if self.hudLogoOverlay then
        self.hudLogoOverlay:delete();
    end;
end;

function TatraTipper:mouseEvent(posX, posY, isDown, isUp, button)
    self.cameras[self.camIndex]:mouseEvent(posX, posY, isDown, isUp, button);
end;

function TatraTipper:keyEvent(unicode, sym, modifier, isDown)
    -- Turn signals
    if InputBinding.isPressed(InputBinding.LEFTTURN) then
        if not self.flashLeft or self.flashWarn then
            self.flashLeft = false;
        end;
        self.flashLeft = not self.flashLeft;
        self.flashRight = false;
        self.flashWarn = false;
        self.timer = 10;
    end;
    if InputBinding.isPressed(InputBinding.RIGHTTURN) then
        if not self.flashRight or self.flashWarn then
            self.flashRight = false;
        end;
        self.flashRight = not self.flashRight;
        self.flashLeft = false;
        self.flashWarn = false;
        self.timer = 10;
    end;
    -- Warn lights
    if InputBinding.isPressed(InputBinding.WARNLIGHTS) then
        if self.flashWarn then
            self.flashLeft = false;
            self.flashRight = false;
        else
            self.rightTurnActive = false;
            self.leftTurnActive = false;
            self.flashLeft = true;
            self.flashRight = true;
        end;
        self.flashWarn = not self.flashWarn;
        self.timer = 10;
    end;
    
    -- Low and high beam lights
    if InputBinding.isPressed(InputBinding.TOGGLE_LIGHTS) then
        if not self.lightsActive then
            self.lightsActive = true;
			self.lowLightsActive = true;
		elseif self.lowLightsActive then
			self.highLightsActive = true;
			self.lowLightsActive = false;
		elseif self.highLightsActive then
			self.highLightsActive = false;
			self.lightsActive = false;
		end;
    end;

    -- Display help text
    if InputBinding.isPressed(InputBinding.HELP) then
        self.helpPanel = not self.helpPanel;
    end;
    
    -- Manual ignition
    if InputBinding.isPressed(InputBinding.IGNITION) then
        self.ignition = not self.ignition;
    end;
end;

function TatraTipper:update(dt)
    -- Handle refueling
    self.doRefuel = self.doRefuel and self.hasRefuelStationInRange;
    if self.doRefuel then
        if not self.refuelSampleRunning and self:getIsActiveForSound() then
            playSample(self.refuelSample, 0, 1, 0);
            self.refuelSampleRunning = true;
        end;

        local refuelSpeed = 0.01;
        local currentFillLevel = self.fuelFillLevel;
        self:setFuelFillLevel(self.fuelFillLevel+refuelSpeed*dt);
        local delta = (self.fuelFillLevel-currentFillLevel);

        if delta <= 0.05 then
            self.doRefuel = false;
        end;

        delta = delta * g_fuelPricePerLiter;

        g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + delta;
        g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + delta;
        g_currentMission.missionStats.money = g_currentMission.missionStats.money - delta;
    else
        if self.refuelSampleRunning then
            stopSample(self.refuelSample);
            self.refuelSampleRunning = false;
        end;
    end;
    
    -- Define input
    if self:getIsActiveForInput() then
        self.input = InputBinding.getAnalogInputAxis(InputBinding.AXIS_FORWARD);
        if InputBinding.isAxisZero(self.input) then
            self.input = InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);
        end;
        -- Turn on the engine when pressing the accelerator
        if math.abs(self.input) > 0.3 and not self.isMotorStarted then
            self.ignition = true;
        end;
    end;
    
    -- Handle engine sounds 
    if self.isMotorStarted then
        if self:getIsActiveForSound() then
            if self.playMotorSound then
                if self.motorSound ~= nil and self.playMotorSoundTime <= self.time then
                    playSample(self.motorSound, 0, self.motorSoundVolume, 0);
                    self.playMotorSound = false;
                    if self.motorSoundRun ~= nil then
                        playSample(self.motorSoundRun, 0, 0.0, 0);
                    end;
                end;
            end;
            if self.compressionSound ~= nil and self.compressionSoundTime < self.time then
                playSample(self.compressionSound, 1, self.compressionSoundVolume, 0);
                setSamplePitch(self.compressionSound, self.compressionSoundPitchOffset);
                self.compressionSoundTime = self.time + 180000;
                self.compressionSoundEnabled = true;
            end;
        end;

        if table.getn(self.wheels) > 0 then
            local alpha = 0.9;
            local roundPerMinute = self.lastRoundPerMinute*alpha + (1-alpha)*(self.motor.lastMotorRpm-self.motor.minRpm);
            self.lastRoundPerMinute = roundPerMinute;
            local roundPerSecond = roundPerMinute / 60;

            if self.motorSound ~= nil then
                setSamplePitch(self.motorSound, math.min(self.motorSoundPitchOffset + 
                  self.motorSoundPitchScale*math.abs(roundPerSecond), self.motorSoundPitchMax));
                if self.motorSoundRun ~= nil then
                    setSamplePitch(self.motorSoundRun, math.min(self.motorSoundRunPitchOffset + 
                      self.motorSoundRunPitchScale*math.abs(roundPerSecond), self.motorSoundRunPitchMax));
                end;
            end;
            
            if self.compressedAirSound ~= nil then
                if self.input == -1 and self.compressedAirSoundEnabled then
                    self.compressedAirSoundEnabled = false;
                end;
                if self.input == 1 and not self.compressedAirSoundEnabled then
                    if self:getIsActiveForSound() then
                        playSample(self.compressedAirSound, 1, self.compressedAirSoundVolume, 0);
                        setSamplePitch(self.compressedAirSound, self.compressedAirSoundPitchOffset);
                    end;
                    self.compressedAirSoundEnabled = true;
                end;
            end;
            
            if self.reverseDriveSound ~= nil then
                if self.movingDirection == -1 then
                    if not self.reverseDriveSoundEnabled and self:getIsActiveForSound() then
                        playSample(self.reverseDriveSound, 0, self.reverseDriveSoundVolume, 0);
                        self.reverseDriveSoundEnabled = true;
                    end;
                else
                    if self.reverseDriveSoundEnabled then
                        stopSample(self.reverseDriveSound);
                        self.reverseDriveSoundEnabled = false;
                    end;
                end;
            end;

            if self.motorSoundRun ~= nil then
                local maxRpm  = self.motorMaxRpmLimit[3];
                if self.input ~= 0 or self.motor.speedLevel ~= 0 then
                    local rpmVolume = Utils.clamp(math.abs(roundPerMinute)/((maxRpm - self.motor.minRpm)), 0.0, 1.0);
                    setSampleVolume(self.motorSoundRun, rpmVolume);
                else
                    local rpmVolume = Utils.clamp(math.abs(roundPerMinute)/((maxRpm - self.motor.minRpm)*2), 0.0, 1.0);
                    setSampleVolume(self.motorSoundRun, rpmVolume);
                end;
            end;
        end;
    end;

    if self:getIsActive() then
        if self.steering ~= nil then
            setRotation(self.steering, 0, self.rotatedTime*self.steeringSpeed, 0);
        end;

        local xt,yt,zt = getTranslation(self.components[1].node);
        local deltaWater = yt-g_currentMission.waterY+2.5;
        if deltaWater < 0 then
            self.isBroken = true;
            g_currentMission:onSunkVehicle();
            if self.isEntered then
                g_currentMission:onLeaveVehicle();
                if self:getIsActiveForSound() then
                    local volume = math.min(1, self.lastSpeed*3600/30);
                    if self.waterSplashSample == nil then
                        self.waterSplashSample = createSample("waterSplashSample");
                        loadSample(self.waterSplashSample, "data/maps/sounds/waterSplash.wav", false);
                    end;
                    playSample(self.waterSplashSample, 1, volume, 0);
                end;
            end;
        end;
        self.showWaterWarning = deltaWater < 2;
    end;

    if self.isEntered then
        if InputBinding.hasEvent(InputBinding.REFUEL) then
            if self.doRefuel then
                self:stopRefuel();
            else
                self:startRefuel();
            end;
        end;

        if self.characterNode ~= nil then
            local cx, cy, cz = getWorldTranslation(self.characterNode);
            local x,y,z = getWorldTranslation(getCamera());
            local dist = Utils.vector3Length(cx-x, cy-y, cz-z);
            if dist < self.characterCameraMinDistance then
                setVisibility(self.characterNode, false);
            else
                setVisibility(self.characterNode, true);
            end;
        end;

        if not g_currentMission.fixedCamera then
            setCamera(self.cameras[self.camIndex].cameraNode);
            self.cameras[self.camIndex]:update(dt);
        else
            if self.tipCamera ~= nil then
                setCamera(self.tipCamera);
            else
                self.cameras[self.camIndex]:resetCamera();
            end;
        end;

        -- Mutes stop sound when engine is off
        if not self.ignition or not self.deactivateOnLeave then
            self.motorStopSoundVolume = 0;
        else
            self.motorStopSoundVolume = self.motorStopSoundVolumeBU;
        end;

        if self.ignition and not self.isMotorStarted then
            self:startMotor();
            self.deactivateOnLeave = false;
            self.steeringEnabled = true;
        elseif not self.ignition and self.isMotorStarted then
            if self.motorStopSoundVolume == 0 then
                self.motorStopSoundVolume = self.motorStopSoundVolumeBU;
            end;

            self:stopMotor();
            self.steeringEnabled = false;
            self.deactivateOnLeave = true;
            setVisibility(self.aiMotorSound, false);

            self:onDeactivateAttachements();
        end;
        if not self.isMotorStarted then
            -- Brakes the wheels upon turning off the engine
            self.motor.brakeForce = 50
            for k,wheel in pairs(self.wheels) do
                setWheelShapeProps(wheel.node, wheel.wheelShape, 0, self.motor.brakeForce, 0);
            end;
        end;

        if self.steeringEnabled then
            local fuelUsed = self.lastMovedDistance*self.fuelUsage;
            self:setFuelFillLevel(self.fuelFillLevel-fuelUsed);

            g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + fuelUsed;
            g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + fuelUsed;
            g_currentMission.missionStats.traveledDistanceTotal = g_currentMission.missionStats.traveledDistanceTotal + self.lastMovedDistance*0.001;
            g_currentMission.missionStats.traveledDistanceSession = g_currentMission.missionStats.traveledDistanceSession + self.lastMovedDistance*0.001;

            local acceleration = 0;
            if g_currentMission.allowSteerableMoving and not self.playMotorSound then
                acceleration = -InputBinding.getAnalogInputAxis(InputBinding.AXIS_FORWARD);
                if InputBinding.isAxisZero(acceleration) then
                    acceleration = -InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);
                end;
                if math.abs(acceleration) > 0.8 then
                    self.motor:setSpeedLevel(0, true)
                end;
                if self.motor.speedLevel ~= 0 then
                    acceleration = 1.0;
                end;
            end;
            
			if self.input == -1 then
				self.motor:setSpeedLevel(0,true)
				self.motor.brakeForce = 0;
			end;
			if self.input == 1 or (self.movingDirection < 0 and self.input == -1) then
				self.motor:setSpeedLevel(0, true)
				self.motor.brakeForce = 50;
			end;
			
			if self.fuelFillLevel == 0 then
                acceleration = 0;
            end;
            
            -- Steering
            local inputAxisX = InputBinding.getAnalogInputAxis(InputBinding.AXIS_SIDE);
            if not InputBinding.isAxisZero(inputAxisX) then
                if inputAxisX < 0 then
                    self.rotatedTime = math.min(-self.maxRotTime*inputAxisX, self.maxRotTime);
                else
                    self.rotatedTime = math.max(self.minRotTime*inputAxisX, self.minRotTime);
                end;
            else
                local rotScale = math.min(1.0/(self.lastSpeed*self.speedRotScale+self.speedRotScaleOffset), 1);
                local inputAxisX = InputBinding.getDigitalInputAxis(InputBinding.AXIS_SIDE);
                if inputAxisX < 0 then
                    self.rotatedTime = math.min(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.maxRotTime);
                elseif inputAxisX > 0 then
                    self.rotatedTime = math.max(self.rotatedTime - dt/1000*inputAxisX*rotScale, self.minRotTime);
                else
                    if self.autoRotateBackSpeed ~= 0 then
                        if self.rotatedTime > 0 then
                            self.rotatedTime = math.max(self.rotatedTime - dt/1000*self.autoRotateBackSpeed*rotScale, 0);
                        else
                            self.rotatedTime = math.min(self.rotatedTime + dt/1000*self.autoRotateBackSpeed*rotScale, 0);
                        end;
                    end;
                end;
            end;
            if self.firstTimeRun then
                WheelsUtil.updateWheelsPhysics(self, dt, self.lastSpeed, acceleration, false, self.requiredDriveMode)
            end;
        end;
        
        -- Rpm limiter (Stefan Geiger)
        if self.motor.speedLevel ~= 0 then
            if InputBinding.isPressed(InputBinding.ACCELERATE) then
                if self.motor.maxRpm[self.motor.speedLevel] <= (self.motorMaxRpmLimit[3] - 25) then
                    self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] + 25;
                end;
            elseif InputBinding.isPressed(InputBinding.DECELERATE) then
                if self.motor.maxRpm[self.motor.speedLevel] >= 25 then
                    self.motor.maxRpm[self.motor.speedLevel] = self.motor.maxRpm[self.motor.speedLevel] - 25;
                end;
            end;
        else
            if InputBinding.isPressed(InputBinding.ACCELERATE) then
                if self.motor.maxRpm[3] <= (self.motorMaxRpmLimit[3] - 25) then
                    self.motor.maxRpm[3] = self.motor.maxRpm[3] + 25;
                end;
            elseif InputBinding.isPressed(InputBinding.DECELERATE) then
                if self.motor.maxRpm[3] >= 25 then
                    self.motor.maxRpm[3] = self.motor.maxRpm[3] - 25;
                end;
            end;
        end;
        
        -- Dashboard
        if self.camIndex == 2 then
            -- RPM
            local rpmValue = 0;
            if self.isMotorStarted then
                rpmValue = self.motor.lastMotorRpm;
            end;
            if rpmValue ~= self.rpmLastValue then
                rpmPerc = rpmValue / self.motorMaxRpmLimit[3];
                self:rotateNeedle(self.rpmNeedle, rpmPerc, self.rpmNeedleMin, self.rpmNeedleMax);
                self.rpmLastValue = rpmValue;
            end;
            -- Speed
            local speedValue = 0;
            if self.isMotorStarted then
                speedValue = self.lastSpeed*self.speedDisplayScale*3600;
            end;
            if speedValue ~= self.speedLastValue then
                speedPerc = speedValue / self.maxSpeed;
                self:rotateNeedle(self.speedNeedle, speedPerc, self.speedNeedleMin, self.speedNeedleMax);
                self.speedLastValue = speedValue;
            end;
            -- Fuel
            local fuelValue = 0;
            if self.isMotorStarted then
                fuelValue = self.fuelFillLevel;
            else
                fuelValue = 0;
            end;
            if fuelValue ~= self.speedLastValue then
                fuelPerc = fuelValue / self.fuelCapacity;
                self:rotateNeedle(self.fuelNeedle, fuelPerc, self.fuelNeedleMin, self.fuelNeedleMax);
                self.fuelLastValue = fuelValue;
            end;
        end;
        
        -- Implement switching
        if InputBinding.hasEvent(InputBinding.SWITCH_IMPLEMENT) then
            local selected = self.selectedImplement;
            local numImplements = table.getn(self.attachedImplements);
            if selected ~= 0 and numImplements > 1 then
                selected = selected+1;
                if selected > numImplements then
                    selected = 1;
                end;
                self:setSelectedImplement(selected);
            end;
        end;

        -- Change camera
        if not g_currentMission.fixedCamera then
            if InputBinding.hasEvent(InputBinding.CAMERA_SWITCH) then
                self.cameras[self.camIndex]:onDeactivate();
                self.camIndex = self.camIndex + 1;
                if self.camIndex > self.numCameras then
                    self.camIndex = 1;
                end;
                self.cameras[self.camIndex]:onActivate();
            end;
        end;
        
        -- Low and high beam lights
        for i=1, self.numLights do
            setVisibility(self.lightObjects[i], self.lightsActive);
		end;
		setVisibility(self.lowLightsObject, self.lowLightsActive);
        setVisibility(self.highLightsObject, self.highLightsActive);

        -- Brake lights                
        if (self.movingDirection*self.lastSpeed*(-self.input)) < -0.001 then
            self.brakeLightActive = true;
        else
            self.brakeLightActive = false;
        end;
        setVisibility(self.brakeLightObject, self.brakeLightActive);
        
        -- Reverse lights
        if self.movingDirection < 0 then
            self.reverseLightActive = true;
        else
            self.reverseLightActive = false;
        end;
        setVisibility(self.reverseLightObject, self.reverseLightActive);

        -- Speed levels
        if InputBinding.hasEvent(InputBinding.SPEED_LEVEL1) then
            self.motor:setSpeedLevel(1, false);
        elseif InputBinding.hasEvent(InputBinding.SPEED_LEVEL2) then
            self.motor:setSpeedLevel(2, false);
        elseif InputBinding.hasEvent(InputBinding.SPEED_LEVEL3) then
            self.motor:setSpeedLevel(3, false);
        end;

        -- Attach trailer
        if InputBinding.hasEvent(InputBinding.ATTACH) then
            self:handleAttachEvent();
        end;

        if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
            self:handleLowerImplementEvent();
        end;

	end;
	
	-- Turn lights
    if self.flashLeft or self.flashRight then
        self.timer = self.timer - dt;
        if self.flashRight or self.flashWarn then
            if self.timer < 0 then
                self.rightTurnActive = not self.rightTurnActive;
            end;
        else
            self.rightTurnActive = false;
        end;
    
        if self.flashLeft or self.flashWarn then
            if self.timer < 0 then
                self.leftTurnActive = not self.leftTurnActive;
            end;
        else
            self.leftTurnActive = false;
        end;
        if self.timer < 0 then
            self.timer = 500;
        end;
    else
        self.rightTurnActive = false;
        self.leftTurnActive = false;
    end;
    setVisibility(self.rightTurnObject, self.rightTurnActive);
    setVisibility(self.leftTurnObject, self.leftTurnActive);
end;

function TatraTipper:onEnter()
    self.isEntered = true;
    self.camIndex = 1;
    self.cameras[self.camIndex]:onActivate();
    if self.ignition then
        self.steeringEnabled = true;
        self:startMotor();
    else
        self.isMotorStarted = false;
        Motorized.stopSounds(self);
        self.steeringEnabled = false;
    end;
    if self.characterNode ~= nil then
        setVisibility(self.characterNode, true);
    end;
    -- Stop motor sound
    setVisibility(self.aiMotorSound, false);
    self:onActivateAttachements();
end;

function TatraTipper:onLeave()
    self.lightsActive = false;
    self.highLightsActive = false;
    self.lowLightsActive = false;
    setVisibility(self.highLightsObject, false);
    setVisibility(self.lowLightsObject, false);

    self.helpPanel = false;
    self.cameras[self.camIndex]:onDeactivate();
    self.motor.brakeForce = 50;

    if self.stopRefuelOnLeave then
        self:stopRefuel();
    end;

    if self.deactivateLightsOnLeave then
        self:setLightsVisibility(false);
    end;

    if self.characterNode ~= nil then
        if self.disableCharacterOnLeave then
            setVisibility(self.characterNode, false);
        else
            setVisibility(self.characterNode, true);
        end;
    end;

    if self.deactivateOnLeave then
        self:stopMotor();
        self.ignition = false;
        self.isMotorStarted = false;
        self:onDeactivateAttachements();
    else
        self.isMotorStarted = true;
        self.ignition = true;
        self.steeringEnabled = false;
        Motorized.stopSounds(self);
        -- Play motor sound
        setVisibility(self.aiMotorSound, true);
        if self.deactivateLightsOnLeave then
            self:onDeactivateAttachementsLights();
        end;
        self:onDeactivateAttachementsSounds();
    end;
    for k,wheel in pairs(self.wheels) do
        setWheelShapeProps(wheel.node, wheel.wheelShape, 0, self.motor.brakeForce, 0);
    end;
    self.isEntered = false;
end;

function TatraTipper:draw()
    setTextColor(1,1,1,1);
   if not self.helpPanel then
      renderText(0.85, 0.86, 0.021, "Num 0: Zeige Hilfe");
   else
      renderText(0.85, 0.86, 0.021, "Num 0: Verberge Hilfe");
   end;
   if self.helpPanel then
        self.hudLogoOverlay:render();
      renderText(0.01, 0.41, 0.025, "Tatra T815-2 TERRNo1 Bergmann HTW");
      renderText(0.01, 0.38, 0.021, "F: Licht aus/normal/Fernlicht");
      renderText(0.01, 0.36, 0.021, "Num 1: Blinker links");
      renderText(0.01, 0.34, 0.021, "Num 3: Blinker rechts");
      renderText(0.01, 0.32, 0.021, "Num 2: Warnlichter");
      renderText(0.01, 0.30, 0.021, "Num +/-: RPM hoch/runter");
      renderText(0.01, 0.28, 0.021, "Num Enter: Zndung");
	  renderText(0.01, 0.26, 0.021, "B: Dosierwalzen");
   end;

    self.hudBaseOverlay:render();

    -- hudBasePoxX = 0.8325;
    -- hudBasePoxY = (1-0.99);
    -- hudBaseWidth = 0.16;
    -- hudBaseHeight = 0.17;
    
    setTextBold(true);
    
	if self.motor.speedLevel ~= 0 then
        renderText(0.86, 0.145, 0.031, ""..string.format("%d",self.motor.maxRpm[self.motor.speedLevel] / 1.25).." RPM");
    else
        renderText(0.86, 0.145, 0.031, ""..string.format("%d",self.motor.maxRpm[3] / 1.25).." RPM");
    end;
    
    if self.lowLightsActive then
        setTextColor(1,0.9,0.23,1);
        renderText(self.hudBasePoxX+0.137, self.hudBasePoxY+0.071, 0.025, "//");
        renderText(self.hudBasePoxX+0.145, self.hudBasePoxY+0.073, 0.035, ")");
    elseif self.highLightsActive then
        setTextBold(false);
        setTextColor(0.0,0.94,1,1);
        renderText(self.hudBasePoxX+0.135, self.hudBasePoxY+0.071, 0.03, "=");
        setTextBold(true);
        renderText(self.hudBasePoxX+0.145, self.hudBasePoxY+0.073, 0.035, ")");
    else
        setTextColor(0.5,0.5,0.5,0.5);
        renderText(self.hudBasePoxX+0.145, self.hudBasePoxY+0.073, 0.035, ")");
    end;
    
    if self.leftTurnActive then
        setTextColor(1,0.68,0,1);
    else
        setTextColor(0.5,0.5,0.5,0.5);
    end;
    renderText(self.hudBasePoxX+0.005, self.hudBasePoxY+0.14, 0.025, "<<");
    
    if self.rightTurnActive then
        setTextColor(1,0.68,0,1);
    else
        setTextColor(0.5,0.5,0.5,0.5);
    end;
    renderText(self.hudBasePoxX+0.135, self.hudBasePoxY+0.14, 0.025, ">>");

    local kmh = self.lastSpeed*self.speedDisplayScale*3600;

    setTextColor(1.0, 1.0, 1.0, 1.0);
    if kmh > 0 and kmh < 100 then
        renderText(self.hudBasePoxX+0.007, self.hudBasePoxY+0.095, 0.06, string.format("%2d", kmh));
    end;
    renderText(self.hudBasePoxX+0.058, self.hudBasePoxY+0.094, 0.06, g_i18n:getText("speedometer"));
    renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.071, 0.03, string.format("%d", g_currentMission.missionStats.money));

    local fuelWarn = 50;
    if self.fuelFillLevel < fuelWarn then
        setTextColor(1,0,0,1);
    end;
    renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.039, 0.03, string.format("%d " .. g_i18n:getText("fluid_unit_long"), self.fuelFillLevel));
    if self.fuelFillLevel < fuelWarn then
        setTextColor(1,1,1,1);
    end;

    setTextBold(false);

    if self.hasRefuelStationInRange and not self.doRefuel and self.fuelFillLevel ~= self.fuelCapacity then
        g_currentMission:addHelpButtonText(g_i18n:getText("Refuel"), InputBinding.REFUEL);
        g_currentMission.hudFuelOverlay:render();
    end;

    if self.showWaterWarning then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_depth_into_the_water"), 0.05, 0.025+0.007);
    end;

    local trailerFillLevel, trailerCapacity = self:getAttachedTrailersFillLevelAndCapacity();
    if trailerFillLevel ~= nil and trailerCapacity ~= nil then
        self:drawGrainLevel(trailerFillLevel+self.fillLevel, trailerCapacity+self.capacity, 101);
    else
        self:drawGrainLevel(self.fillLevel, self.capacity, 101);
    end;
    if table.getn(self.attachedImplements) > 1 then
        g_currentMission:addHelpButtonText(g_i18n:getText("Change_tools"), InputBinding.SWITCH_IMPLEMENT);
    end;
end;

function TatraTipper:drawGrainLevel(level, capacity, warnPercent)

    local percent = 0;
    if capacity > 0 then
        percent = level/capacity*100;
    end;
    setTextBold(true);

    if percent >= warnPercent then
        setTextColor(1.0, 0.0, 0.0, 1.0);
    else
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;

    renderText(self.hudBasePoxX+0.031, self.hudBasePoxY+0.006, 0.03, string.format("%d(%d%%)", level, percent));
    if percent >= warnPercent then
        setTextColor(1.0, 1.0, 1.0, 1.0);
    end;
    setTextBold(false);
end;

function TatraTipper:startMotor()

    if not self.isMotorStarted then
        self.isMotorStarted = true;

        self.reverseDriveSoundEnabled = false;

        local motorSoundOffset = 0;
        if self.motorStartSound ~= nil and self:getIsActiveForSound() then
            playSample(self.motorStartSound, 1, self.motorStartSoundVolume, 0);
            setSamplePitch(self.motorStartSound, self.motorStartSoundPitchOffset);
            motorSoundOffset = getSampleDuration(self.motorStartSound);
        end;

        self.playMotorSound = true;
        self.playMotorSoundTime = self.time+motorSoundOffset;
        self.playCompressionSoundTime = self.time+motorSoundOffset;
        self.compressionSoundTime = self.time + 180000;

        self.lastRoundPerMinute=0;

        Utils.setEmittingState(self.exhaustParticleSystems, true)
    else
        self.playMotorSound = true;
    end;
end;

function TatraTipper:stopMotor()

    self.isMotorStarted = false;
    Motorized.stopSounds(self);

    if self:getIsActiveForSound() then
        if self.motorStopSound ~= nil then
            setSamplePitch(self.motorStopSound, self.motorStopSoundPitchOffset);
            playSample(self.motorStopSound, 1, self.motorStopSoundVolume, 0);
        end;
    end;

    self.motor:setSpeedLevel(0, false);

    Utils.setEmittingState(self.exhaustParticleSystems, false)
end;

function TatraTipper:stopSounds()

    self.playMotorSound = false;
    if self.motorSound ~= nil then
        stopSample(self.motorSound);
    end;

    self.playMotorRunSound = false;
    if self.motorSoundRun ~= nil then
        stopSample(self.motorSoundRun);
    end;

    if self.motorStartSound ~= nil then
        stopSample(self.motorStartSound);
    end;

    if self.compressionSoundEnabled then
        stopSample(self.compressionSound);
        self.compressionSoundEnabled = false;
    end;
end;

function TatraTipper:startRefuel()

    if self.hasRefuelStationInRange then
        self.doRefuel = true;
    end;
end;

function TatraTipper:stopRefuel()

    self.doRefuel = false;

    if self.refuelSampleRunning then
        stopSample(self.refuelSample);
        self.refuelSampleRunning = false;
    end;
end;


function TatraTipper:setFuelFillLevel(newFillLevel)
    self.fuelFillLevel = math.max(math.min(newFillLevel, self.fuelCapacity), 0);
end;


function TatraTipper:rotateNeedle(object, percent, min, max) 
    angle = min + (max - min) * percent;
    setRotation(object, self.PI, angle, self.PI_2);
end;